/* 
 * METRIC-Application:
 * Corner / 90^o bend in a slab waveguide
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"metric.h"

#define Pol    TE    // light polarization
#define CPnb   1.0   // background refractive index
#define CPng   3.4   // core refractive index
#define CPw    0.22  // core width /mum
#define Wavel  1.55  // vacuum wavelength /mum

#define CWx    7.0   // computational window, 
#define CWz    7.0   // distances from the coner center 
#define NumMx  100   // number of spectral terms in the field discretization
#define NumMz  100   //

#define DWx    4.0   // display window, distances from the corner center 
#define DWz    4.0   //

/* ------------------------------------------------------------------------ */ 

/* the corner configuration */
Circuit cornerdef()
{
	Circuit cr(1, 1);
	cr.hx(0) = -CPw/2.0;
	cr.hx(1) =  CPw/2.0;
	cr.hz(0) = -CPw/2.0;
	cr.hz(1) =  CPw/2.0;
	cr.n(2, 0) = CPnb; cr.n(2, 1) = CPng; cr.n(2, 2) = CPnb;
	cr.n(1, 0) = CPng; cr.n(1, 1) = CPng; cr.n(1, 2) = CPnb;
	cr.n(0, 0) = CPnb; cr.n(0, 1) = CPnb; cr.n(0, 2) = CPnb;
	cr.lambda = Wavel;
	return cr;
}

/* ------------------------------------------------------------------------ */ 

/* corner simulation */
int main()
{
	fprintf(stderr, "\nWaveguide corner:\n");	
	fprintf(stderr, "----------------\n");	

	fprintf(stderr, "lambda = %g mum\n", Wavel);
	fprintf(stderr, "n_b    = %g\n", CPnb);
	fprintf(stderr, "n_g    = %g\n", CPng);
	fprintf(stderr, "w      = %g mum\n\n", CPw);

	// the structure under investigation 
	Circuit cr = cornerdef();
	// vertical computational window
	Interval vcw(-CWx, CWx);
	// horizontal computational window
	Interval hcw(-CWz, CWz);
	// QUEP setup 
	QuepField qfld(cr, Pol, vcw, NumMx, hcw, NumMz);

	// input: the fundamental mode in the left channel 
	qfld.input(LEFT, CC1, 0);
	// calculate the light propagation along the corner
	qfld.quepsim(); 

	fprintf(stderr, "\nScattered fundamental mode amplitudes:\n"); 
	Complex a; 
	double p;
	a = qfld.Aout(LEFT, 0);
	p = qfld.Pout(LEFT, 0);
	fprintf(stderr, "R = %g + i %g,  |R|^2 = %g\n", a.re, a.im, p);
	a = qfld.Aout(TOP, 0);
	p = qfld.Pout(TOP, 0);
	fprintf(stderr, "T = %g + i %g,  |T|^2 = %g\n", a.re, a.im, p);
	
	double pb, pf, pd, pu;
	pb = qfld.Pout(LEFT);
	pf = qfld.Pout(RIGHT);
	pd = qfld.Pout(BOTTOM);
	pu = qfld.Pout(TOP);
	
	fprintf(stderr, "Total scattered power:\n");
	fprintf(stderr, "P_back    = %g\n", pb);
	fprintf(stderr, "P_forw    = %g\n", pf);
	fprintf(stderr, "P_down    = %g\n", pd);
	fprintf(stderr, "P_up      = %g\n", pu);
	fprintf(stderr, "Sum P_out = %g\n", pb+pf+pd+pu);
	
	pb = qfld.Pgout(LEFT);
	pu = qfld.Pgout(TOP);
	fprintf(stderr, "Total scattered guided power:\n");
	fprintf(stderr, "P_back    = %g\n", pb);
	fprintf(stderr, "P_up      = %g\n\n", pu);

	char pc = polchr(Pol);
	Fcomp fc = principalcomp(Pol);
	Mlop_Print = NO;
	Mlop_Colour = YES;

	qfld.adjustphase(0.0, -CPw/8.0);
	qfld.viewer(-DWx, DWx, -DWz, DWz, 150, 150, 't', pc); 
	qfld.plot(fc, REP, -DWx, DWx, -DWz, DWz, 150, 150, 't', pc); 
	qfld.plot(fc, IMP, -DWx, DWx, -DWz, DWz, 150, 150, 't', pc); 
	qfld.plot(fc, MOD, -DWx, DWx, -DWz, DWz, 150, 150, 't', pc);
	qfld.phasemap(fc, -DWx, DWx, -DWz, DWz, 150, 150, 't', pc);
	qfld.movie(-DWx, DWx, -DWz, DWz, 150, 150, 30, 't', pc);             
	qfld.fplot(fc, -DWx, DWx, -DWz, DWz, 150, 150, 't', pc); 
	qfld.fmovie(-DWx, DWx, -DWz, DWz, 150, 150, 30, 't', pc);             

	fprintf(stderr, "\nOk.\n");
	return 0; 
}
