/* 
 * METRIC-Application:
 * A waveguide with a single hole in it, QUEP simulation
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"metric.h"

#define Pol     TE     // light polarization
#define HPnc    1.0    // cover refractive index
#define HPnf    2.00   // film refractive index
#define HPns    1.45   // substrate refractive index
#define HPt     0.200  // film thickness /mum
#define HPs     0.200  // width of the hole in the film /mum
#define Wavel   0.633  // vacuum wavelength /mum

#define CWbot   -5.0   // boundaries of the computational window 
#define CWtop    5.0   // 
#define CWleft  -5.0   //  
#define CWright  5.0   //  
#define NumMx    120   // number of spectral terms in the field discretization
#define NumMz    120   // 

#define DWx0    -2.0   // display window
#define DWx1     2.0
#define DWz0    -2.0
#define DWz1     2.0


/* waveguide with a central hole */
Circuit circdef()
{
	Circuit hwg(1, 1);
	hwg.hx(0) = 0.0;
	hwg.hx(1) = HPt;
	hwg.hz(0) = -HPs/2.0;
	hwg.hz(1) = HPs/2.0;
	hwg.n(2, 0) = HPnc; hwg.n(2, 1) = HPnc; hwg.n(2, 2) = HPnc;
	hwg.n(1, 0) = HPnf; hwg.n(1, 1) = HPnc; hwg.n(1, 2) = HPnf;
	hwg.n(0, 0) = HPns; hwg.n(0, 1) = HPns; hwg.n(0, 2) = HPns;
	hwg.lambda = Wavel;
	return hwg;
}

/* simulate the incidence of the fundamental guided mode */ 
int main()
{

	// the waveguide with hole 
	Circuit hwg = circdef();

	// vertical computational window
        Interval vcw(CWbot, CWtop);
        // horizontal computational window
        Interval hcw(CWleft, CWright);

	// QUEP setup + spectral discretizaion
	QuepField qfld(hwg, Pol, vcw, NumMx, hcw, NumMz);

        // input: the fundamental mode in the left channel
	qfld.input(LEFT, CC1, 0);

        // calculate the light propagation across the hole
        qfld.quepsim();

	Complex a; 
	double p;
	fprintf(stderr, "\nScattered fundamental mode amplitudes:\n");
	a = qfld.Aout(LEFT, 0);
	p = qfld.Pout(LEFT, 0);
	fprintf(stderr, "R = %g + i %g,  |R|^2 = %g\n", a.re, a.im, p);
	a = qfld.Aout(RIGHT, 0);
	p = qfld.Pout(RIGHT, 0);
	fprintf(stderr, "T = %g + i %g,  |T|^2 = %g\n", a.re, a.im, p);

	double pb, pf, pd, pu;
        pb = qfld.Pout(LEFT);
        pf = qfld.Pout(RIGHT);
        pd = qfld.Pout(BOTTOM);
        pu = qfld.Pout(TOP);
        fprintf(stderr, "Total scattered power:\n");
        fprintf(stderr, "P_back      = %g\n", pb);
        fprintf(stderr, "P_forw      = %g\n", pf);
        fprintf(stderr, "P_down      = %g\n", pd);
        fprintf(stderr, "P_up        = %g\n", pu);
        fprintf(stderr, "Sum P_out   = %g\n", pb+pf+pd+pu);

        pb = qfld.Pgout(LEFT);
        pf = qfld.Pgout(RIGHT);
        fprintf(stderr, "Total scattered guided power:\n");
        fprintf(stderr, "P_back      = %g\n", pb);
        fprintf(stderr, "P_forw      = %g\n\n", pf);

	Fcomp fc = principalcomp(Pol);
	char pc = polchr(Pol);
	double dwl, dwr, dwb, dwt;
	dwl = DWz0; 
	dwr = DWz1;
	dwb = DWx0;
	dwt = DWx1;
	qfld.adjustphase(0.0, 0.0);
        qfld.viewer(dwb, dwt, dwl, dwr, 150, 150, 't', pc);
        qfld.plot(fc, REP, dwb, dwt, dwl, dwr, 150, 150, 't', pc);
        qfld.movie(dwb, dwt, dwl, dwr, 150, 150, 30, 't', pc);
        qfld.fplot(fc, dwb, dwt, dwl, dwr, 150, 150, 't', pc);
        qfld.fmovie(dwb, dwt, dwl, dwr, 150, 150, 30, 't', pc);

        fprintf(stderr, "\nOk.\n");
	return 0;
}
