/* 
 * METRIC-Application:
 * laterally buried strip waveguide, low core / substrate contrast, 
 * VEIMS mode analysis 
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"metric.h"

#define Pol   TE     // light polarization
#define Wgpns 1.99   // substrate refractive index
#define Wgpnf 1.45   // lateral cover refractive index
#define Wgpnc 1.0    // cover: air
#define Wgpcc 7.5e-3 // core refractive index; contrast vs. substrate 
#define Wgpw  20.0   // strip width
#define Wgph  10.0   // strip height
#define Wgpl  2.0    // vacuum wavelength 

/* plots: display window */
Rect Display(-Wgph, -1.1*Wgpw, Wgph*1.5, 1.1*Wgpw);

/* waveguide definition */
WgCrs wgdef()
{
	WgCrs g(1, 1);

	g.hx(0) = 0.0;
	g.hx(1) = Wgph;
	g.hy(0) = -Wgpw/2.0;
	g.hy(1) =  Wgpw/2.0;

	g.n(0,0) = Wgpns;
	g.n(0,1) = Wgpns;
	g.n(0,2) = Wgpns;
	g.n(1,0) = Wgpnf;
	g.n(1,1) = Wgpns+Wgpcc;
	g.n(1,2) = Wgpnf;
	g.n(2,0) = Wgpnc;
	g.n(2,1) = Wgpnc;
	g.n(2,2) = Wgpnc;

	g.lambda = Wgpl;

	return g;
}

/* VEIMS mode analysis, mode profile plots */
int main()
{
	// waveguide definition, refractive index plot
	WgCrs wg = wgdef();
	wg.plot(Display, '0', '0');

	// containers for modes and mode arrays
	EIMode m;	
	EIModeArray ma;

	// invoke the VEIMS mode solver, found modes -> ma 	
	veims(wg, Pol, ma);
	
	for(int j=0; j<=ma.num-1; ++j)
	{
		m = ma(j);
		fprintf(stderr, "[%d]  T%c_(%d, %d): ", 
		        j, polCHR(m.pol), m.vmp.ord, m.hmp.ord);
		fprintf(stderr, "n_eff = %g  ", m.neff);
		fprintf(stderr, "beta = %g\n", m.beta);

		// plots of the constituting 1-D mode profiles
		m.vmp.plot('v', dig1(j)); 
		m.hmp.plot('h', dig1(j)); 

		// mode profile plots, different variants		
		Fcomp cp = principalcomp(m.pol);
		m.plot(cp, ORG, Display, 150, 150, dig10(j), dig1(j), 'C');
		m.plot(cp, SQR, Display, 150, 150, dig10(j), dig1(j), 'C');
		m.plot(cp, ORG, Display, 75, 75, dig10(j), dig1(j), 'S');
		m.plot(cp, ORG, Display, 150, 150, dig10(j), dig1(j), 'I');
		m.fplot(cp, Display, 150, 150, dig10(j), dig1(j));
		m.acplot(Display, 75, 75, dig10(j), dig1(j));

		// generate a Matlab mode profile viewer
		m.viewer(Display, 150, 150, dig10(j), dig1(j));

		// evaluate the power fraction in the core region
		double nrm, frac;
		Rect core(0.0, -Wgpw/2.0, Wgph, Wgpw/2.0); 
		nrm = m.power();
		frac = 0.5*(m.recint(EX, HY, core)-m.recint(EY, HX, core));
		fprintf(stderr, "N = %g, F = %g, PF = %g\n", nrm, frac, frac/nrm);
		fprintf(stderr, "\n");
	}
	fprintf(stderr, "Ok.\n");
	return 0;
}
